(define-module (ui ahttpd power ajax)
  :use-module (alterator ajax)
  :use-module (alterator algo)
  :use-module (alterator woo)
  :export (init))

(define (update-checkbox-block checkname availability flag . fieldlist)
  (form-update-value checkname (and availability flag))
  (form-update-activity (append fieldlist (list checkname))
                        availability))

(define (ui-read)
  (form-update-value "shutdown_type" "none")
  (let* ((param-list (woo-call "/ahttpd-power/read_power"))
         (params (cond-car param-list))
         (is-email-available (woo-get-option params 'email_available))
         (can-shutdown (woo-get-option params 'can_shutdown))
         (can-reboot (woo-get-option params 'can_reboot))
         (can-suspend (woo-get-option params 'can_suspend))
         (can-hibernate (woo-get-option params 'can_hibernate)))
    (form-update-value-list
     '("shutdown_time" "reboot_time" "suspend_time" "hibernate_time"
       "email_addr")
     param-list)
    (update-checkbox-block "shutdown_cron" can-shutdown
                           (woo-get-option params 'shutdown_cron)
                           "shutdown_time" "#poweroff")
    (update-checkbox-block "reboot_cron" can-reboot
                           (woo-get-option params 'reboot_cron)
                           "reboot_time" "#reboot")
    (update-checkbox-block "suspend_cron" can-suspend
                           (woo-get-option params 'suspend_cron)
                           "suspend_time" "#suspend")
    (update-checkbox-block "hibernate_cron" can-hibernate
                           (woo-get-option params 'hibernate_cron)
                           "hibernate_time" "#hibernate")
    (update-checkbox-block "notify_email" is-email-available
                           (woo-get-option params 'notify_email)
                           "email_addr")))

(define (ui-poweroff)
  (woo-call "/ahttpd-power/poweroff"))

(define (ui-reboot)
  (woo-call "/ahttpd-power/reboot"))

(define (ui-suspend)
  (woo-call "/ahttpd-power/suspend")
  (form-set-timeout (lambda()
                      (form-update-visibility "suspend_message" #f))
                    1))

(define (ui-hibernate)
  (woo-call "/ahttpd-power/hibernate")
  (form-set-timeout (lambda()
                      (form-update-visibility "hibernate_message" #f))
                    1))

(define (ask-action message-name message action)
  (if (form-confirm (_ message) (_"Question") (_ "Yes") (_ "No"))
    (begin (form-update-visibility message-name #t)
           (form-set-timeout action 1))))

(define (ui-write)
  (catch/message-and-ignore
   (lambda()
     (apply
      woo-call "/ahttpd-power/write_power"
               'language (form-value "language")
               (form-value-list '("shutdown_time" "shutdown_cron"
                                  "reboot_time" "reboot_cron"
                                  "suspend_time" "suspend_cron"
                                  "hibernate_time" "hibernate_cron"
                                  "notify_email" "email_addr")))
     (case (string->symbol (or (form-value "shutdown_type") ""))
       ((poweroff)
        (ask-action "poweroff_message"
                    "Do you really want to shutdown the system?"
                    ui-poweroff))
       ((reboot)
        (ask-action "reboot_message"
                    "Do you really want to reboot the system?"
                    ui-reboot))
       ((suspend)
        (ask-action "suspend_message"
                    "Do you really want to suspend the system?"
                    ui-suspend))
       ((hibernate)
        (ask-action "hibernate_message"
                    "Do you really want to hibernate the system?"
                    ui-hibernate)))))
  (ui-read))

(define (init)
  (form-update-visibility "poweroff_message" #f)
  (form-update-visibility "reboot_message" #f)
  (form-update-visibility "suspend_message" #f)
  (form-update-visibility "hibernate_message" #f)
  (ui-read)
  (form-bind "apply_button" "click" ui-write)
  (form-bind "reset_button" "click" ui-read))
